/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

/*
 * Created on October 21, 2005
 *
 */
package com.sunlabs.archsim.fleet.ships.tokenadder;
import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;




/**
 * @author mh193489
 * 
 * Adder ship for fleet.
 * Accepts integer on In terminal and outputs the cumulative sum on Out terminal.
 */
public class TokenAdder extends Component {
		
	InputTerminal in1;
	OutputTerminal ackIn1;

        InputTerminal in2;
	OutputTerminal ackIn2;

	OutputTerminal out;
	InputTerminal ackOut;

        
        OutputTerminal tokenIn1;
	InputTerminal ackTokenIn1;

        OutputTerminal tokenIn2;
	InputTerminal ackTokenIn2;

	InputTerminal tokenOut;
	OutputTerminal ackTokenOut;

	/** accumulator action hidden in the ship */
	AdderAction adder;
	

	public TokenAdder(String name, Parameters params) {
        super(name, null);
        
		// process the parameters
		if (params != null) {
			for (int i= 0; i<params.size(); i++) {
				String pName= params.getName(i);
				String sValue= params.getValue(i);
		
				director.error("Component: " + name + ", id= " + getID() 
						+ ": Attribute " + pName + " with value " + sValue
						+ " is ignored");					
		
			} // for
		} // if params != null
		        
    	
        construct(name);				
	} // Adder

	
	public TokenAdder(String name) {
    	super(name, null);
        construct(name);				
	} // Adder
	
    private void construct(String name) {
		
        // create terminals for the Adder component
		in1= new InputTerminal(AdderDefaults.IN1, this);
        ackIn1= new OutputTerminal(AdderDefaults.ACK_IN1, this);

        in2= new InputTerminal(AdderDefaults.IN2, this);
        ackIn2= new OutputTerminal(AdderDefaults.ACK_IN2, this);

        out= new OutputTerminal(AdderDefaults.OUT, this);
        ackOut= new InputTerminal(AdderDefaults.ACK_OUT, this);

        // create terminals for the Adder component
		tokenIn1= new OutputTerminal(AdderDefaults.TOKEN_IN1, this);
        ackTokenIn1= new InputTerminal(AdderDefaults.ACK_TOKEN_IN1, this);

        tokenIn2= new OutputTerminal(AdderDefaults.TOKEN_IN2, this);
        ackTokenIn2= new InputTerminal(AdderDefaults.ACK_TOKEN_IN2, this);

        tokenOut= new InputTerminal(AdderDefaults.TOKEN_OUT, this);
        ackTokenOut= new OutputTerminal(AdderDefaults.ACK_TOKEN_OUT, this);                
                
        // create the action performing the adding
		adder= new AdderAction(AdderDefaults.ADDER, this);

        // attach terminals to the action
        adder.attach(in1);
		adder.attach(ackIn1);
		adder.attach(in2);
		adder.attach(ackIn2);
		adder.attach(out);
		adder.attach(ackOut);

                adder.attach(tokenIn1);
		adder.attach(ackTokenIn1);
		adder.attach(tokenIn2);
		adder.attach(ackTokenIn2);
		adder.attach(tokenOut);
		adder.attach(ackTokenOut);
                
    }
        

	
	public InputTerminal getIn1() {
		return in1;
	}
	
	public OutputTerminal getAckIn1() {
		return ackIn1;
	}

        public InputTerminal getIn2() {
		return in2;
	}
	
	public OutputTerminal getAckIn2() {
		return ackIn2;
	}
	
	public OutputTerminal getOut() {
		return out;
	}
	
	public InputTerminal getAckOut() {
		return ackOut;
	}
	
	/** 
	 * Get a terminal by name.
	 * Return null if no such terminal exists.
	 */
	public Terminal getTerminal(String name) {
            
                /* data terminals */
            
		if (name.equals(in1.getBaseName()) 
				|| name.equals(in1.getBaseAlias()))
			return in1;
		else if (name.equals(ackIn1.getBaseName()) 
				|| name.equals(ackIn1.getBaseAlias()))
			return ackIn1;
                else if (name.equals(in2.getBaseName()) 
				|| name.equals(in2.getBaseAlias()))
			return in2;
		else if (name.equals(ackIn2.getBaseName()) 
				|| name.equals(ackIn2.getBaseAlias()))
			return ackIn2;
		else if (name.equals(out.getBaseName())
				|| name.equals(out.getBaseAlias()))
			return out;
		else if (name.equals(ackOut.getBaseName())
				|| name.equals(ackOut.getBaseAlias()))
			return ackOut;
		
                /* token terminals */

                else if (name.equals(tokenIn1.getBaseName()) 
				|| name.equals(tokenIn1.getBaseAlias()))
			return tokenIn1;
		else if (name.equals(ackTokenIn1.getBaseName()) 
				|| name.equals(ackTokenIn1.getBaseAlias()))
			return ackTokenIn1;
                else if (name.equals(tokenIn2.getBaseName()) 
				|| name.equals(tokenIn2.getBaseAlias()))
			return tokenIn2;
		else if (name.equals(ackTokenIn2.getBaseName()) 
				|| name.equals(ackTokenIn2.getBaseAlias()))
			return ackTokenIn2;
		else if (name.equals(tokenOut.getBaseName())
				|| name.equals(tokenOut.getBaseAlias()))
			return tokenOut;
		else if (name.equals(ackTokenOut.getBaseName())
				|| name.equals(ackTokenOut.getBaseAlias()))
			return ackTokenOut;
                
                else 
			return null;
			
	} // getTerminal
	
} // Adder;


